<?php
/******************************************************************************
* Subs-TimeZone.php                                                           *
*******************************************************************************
* For SMF: Simple Machines Forum                                              *
* Open-Source Project Inspired by Zef Hemel (zef@zefhemel.com)                *
* =========================================================================== *
* Software Version:           SMF 1.1.5+                                      *
* File Version:               1.1.0                                           *
* Software by:                DIN1031                                         *
* Copyright 2008 by:	     		DIN1031									                        *
* Support, News, Updates at:  http://mods.simplemachines.org                  *
*******************************************************************************
* This program is free software; you may redistribute it and/or modify it     *
* under the terms of the provided license as published by SMF.                *
*                                                                             *
* This program is distributed in the hope that it is and will be useful,      *
* but WITHOUT ANY WARRANTIES; without even any implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                        *
******************************************************************************/

if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file contain function for the DST calculation and setting
	array loadTimeZones(array tz = true, bool ignore = true)
		- load data into the $context['timezones']['TZ'] and $context['timezones']['DTS']
		- tz load specific timezones, can be also a single string value
		- ignore will not show disabled timezone

	bool load_masterZones()
		- Create a list of the Master Zones and count them

	array searchTimezones(string search, string type = 'all', bool firstOnly = false, bool $ignore = true) {
		- serach is the item you search for, you can use sql like statments for this
		- Follow types are supported
			'all' => Search for all
			'zone' => search only the zone
			'code' => iso code only
			'master' => master zone only
			'sub' => sub zone only
			'country' => country name only
			'short' => short zone search only
			'replaceName' => Name that placed by the Admin ;)
			'designation' => serach -12/+12 time offset
		- firstOnly will output only the first founded item
		- ignore will not show disabled timezone
		- Return the timezones array for the search

	int zoneTimestamp(int logTime = 0, string zone = false) {
		- This function return a zone based timestamp
		- Can calculate a time for each given timezone
		- "zone = false" will return the user based or forum default timezone
		- The logTime is a normal timestamp, on empty value it use the current one

	int utcTimestamp(int logTime = 0)
		- return the calculated utc timestamp

	array calculateZoneDtsData(string zone, int utcTimestamp = false)
		- Return informations about the current timeoffset and the next calculated DST timestamp
		- You can use a diffrent UTC timestamp, only for test reasons

	array dtsTimestamp($in, $on, $hour, $min, $type, $last_seconds, $utcSeconds, $year = false)
		- This function recalculate the given rules to the dst timestamp

	int timezone_timestamp($use_user_offset = true, $timestamp = null)
		- !!!!

	void update_standardTimeZone()
		- Update the Standard Timezone DST

	void update_memberTimeZone(int memID, string timezone)
		- Update the Member Timezone DST

	int local_userTime(int memID)
		- This load the local user time and make checkup of the offsets and correct them if needed
		- return the current timestamp in the SMF Format (timeformat)

	string return_selectField(string zone = '', string offsetTime = 0)
		- return the current short zone in the correct dst version
		
	string return_selectField($tz_data)
		- This is a helper to select the correct timezone on the pull down
*/

//Load the huge $context['timezones']['TZ'] and $context['timezones']['DTS']
function loadTimeZones($tz = true, $ignore = true)
{
	global $context, $modSettings, $db_prefix, $func;

	if(empty($tz) && $tz !== true)
		return true;

	if(!isset($context['timezones']))
		$context['timezones'] = array('TZ' => array(), 'DTS' => array(), 'full' => false);
	elseif(!isset($context['timezones']['TZ']))
		$context['timezones'] += array('TZ' => array(), 'DTS' => array(), 'full' => false);
	elseif($context['timezones']['full'])
		return true;

	//Prepare SQL String, detetect allready loaded zones
	$sql = '';
	if(!empty($tz) && is_array($tz))
	{
		$tz = array_unique($tz);
		if(!empty($context['timezones']['TZ']))
			$tz = array_diff($tz, array_keys($context['timezones']['TZ']));
		if(empty($tz))
			return true;

		if(count($tz) > 1)
			$sql = "WHERE zone IN ('".implode("', '", $tz)."')";
		else
			$sql = "WHERE zone = '".current($tz)."' LIMIT 1";
	}
	elseif(!empty($tz) && $tz !== true)
	{
		if(isset($context['timezones'][$tz]))
			return true;
		$sql = "WHERE zone = '$tz' LIMIT 1";
	}
	else
		$context['timezones']['full'] = $tz;

	//Ignore the Disabled inserts =)
	if($ignore === true)
		$sql = empty($sql) ? "WHERE disabled = 0" : str_replace("WHERE", "WHERE disabled = 0 AND", $sql);

	//Grap the needed informations and store them into the $context
	$load_rules = array();
	$zone = db_query("SELECT * FROM {$db_prefix}timezone $sql", __FILE__, __LINE__);
	while($row = mysql_fetch_assoc($zone))
	{
		$row['replaceName'] = empty($row['replaceName']) ? str_replace("_", " ", $row['zone']) : $row['replaceName'];
		$row['master_zone'] = str_replace("_", " ", $row['master_zone']);
		$row['sub_zone'] = str_replace("_", " ", $row['sub_zone']);
		$context['timezones']['TZ'][$row['zone']] = $row;
		if(!empty($row['rule']))
			$load_rules[$row['rule']] = $row['rule'];
	}
	mysql_free_result($zone);

	//Any DTS Ruls to load?
	if(!empty($load_rules))
	{
		if(count($load_rules) > 1)
			$sql = "WHERE rule IN ('".implode("', '", $load_rules)."')";
		else
			$sql = "WHERE rule = '".current($load_rules)."' LIMIT 1";
		$dts = db_query("SELECT * FROM {$db_prefix}timezone_dtsrules $sql", __FILE__, __LINE__);
		while($row = mysql_fetch_assoc($dts))
			$context['timezones']['DTS'][$row['rule']] = $row;
		mysql_free_result($dts);
	}

	//Say i've something done *g*
	return !empty($context['timezones']);
}

//Create a list of the Master Zones and count them 8)
function load_masterZones()
{
	global $context, $modSettings, $db_prefix;

	if(!isset($context['timezones']['master']))
	{
		$context['timezones']['master'] = array();
		$context['timezones']['master_counts'] = array();
		$context['timezones']['zone_count'] = 0;
	}
	else
		return true;

	$masterzones = db_query("
		SELECT count(*) as count, master_zone
		FROM {$db_prefix}timezone
		GROUP BY master_zone", __FILE__, __LINE__);

	while($row = mysql_fetch_assoc($masterzones)) {
		$context['timezones']['master'][] = $row['master_zone'];
		$context['timezones']['master_counts'][$row['master_zone']] = $row['count'];
		$context['timezones']['zone_count'] += $row['count'];
	}
	mysql_free_result($masterzones);

	return true;
}

/*
Return the Timezones array for a search
$types Supported
	'all' => Search for all
	'zone' => search only the zone
	'code' => iso code only
	'master' => master zone only
	'sub' => sub zone only
	'country' => country name only
	'short' => short zone search only
	'replaceName' => Name that placed by the Admin ;)
	'designation' => serach -12/+12 time offset
$search is like based so you can use % for this
$firstOnly return only one result ;)
*/
function searchTimezones($search, $type = 'all', $firstOnly = false, $ignore = true)
{
	global $func, $db_prefix;

	//Search Checkup
	if(empty($search))
		return array();
	else
	{
		$search = $func['htmlspecialchars']($search, ENT_QUOTES);
		$search = str_replace(" ", "%", $search);
	}
	//Current Supported Fields
	$searchFields = array('zone' => 'zone', 'code' => 'iso_code', 'master' => 'master_zone', 'sub' => 'sub_zone', 'replaceName' => 'replaceName', 'country' => 'country_name', 'short' => 'short_zone', 'designation' => 'utc');
	$sql = array();
	if(!isset($searchFields[$type]))
	{
		foreach($searchFields as $field)
			$sql[] = $field." LIKE '".$search."'";
	}
	else
		$sql[] = $searchFields[$type]." LIKE '".$search."'";

	$zones = array();
	//Search the time zones
	$zone = db_query("
		SELECT zone
		FROM {$db_prefix}timezone
		WHERE (".implode(" OR ", $sql).")".($ignore ? "
		 AND disabled = 0" : "").($firstOnly ? "
		 LIMIT 1" : ""), __FILE__, __LINE__);

	while($z = mysql_fetch_assoc($zone))
		$zones[current($z)] = current($z);

	mysql_free_result($zone);

	return $zones;
}

//This is only a test Function!
function zoneTimestamp($logTime = 0, $zone = false)
{
	global $context, $user_info, $modSettings;

	if($zone === false && !empty($user_info['timezone']))
		$zone = $user_info['timezone'];
	elseif($zone === false && !empty($modSettings['timezone_standard']))
		$zone = $modSettings['timezone_standard'];

	if($logTime <= 0)
		$utcLogTime = utcTimestamp();
	else
		$utcLogTime = utcTimestamp($logTime);

	loadTimeZones($zone);
	//Return error on invalid insert!
	if(!isset($context['timezones']['TZ'][$zone]))
		return false;

	//Correct the timezone :)
	$dts = calculateZoneDtsData($zone, $utcLogTime);

	//Everthing avaible for correct Time :D
	return $utcLogTime + $context['timezones']['TZ'][$zone]['seconds'] + $dts['saveSeconds'];
}

//The gmdate sometimes not work on windows, this will return the utc without dts 8)
function utcTimestamp($logTime = 0)
{
	global $modSettings;

	if($logTime <= 0)
		$logTime = time();

	$dayTimeSave = date("I", $logTime);
	$currentGMT = date("Z", $logTime);

	if(!empty($modSettings['timezones_utcFix']))
	 $logTime += $modSettings['timezones_utcFix'];

	//This is needed on server who not change the gmt for dts
	if(!empty($modSettings['timezones_dtsFix']))
		$dayTimeSave = $dayTimeSave * 3600;
	else
		$dayTimeSave = 0;

	return ($logTime - $currentGMT - $dayTimeSave);
}

//Callculate the next DTS update of this zone :)
function calculateZoneDtsData($zone, $utcTimestamp = false)
{
	global $context, $modSettings, $db_prefix;

	loadTimeZones($zone);
	if(!isset($context['timezones']['TZ'][$zone]))
		return false;

	$return = array(
		'nextUpdate' => 0,
		'saveSeconds' => 0,
		'saveHours' => 0,
		'offsetSeconds' => $context['timezones']['TZ'][$zone]['seconds'],
		'offsetHours' => $context['timezones']['TZ'][$zone]['hours'],
	);

	if(empty($context['timezones']['TZ'][$zone]['dts']))
		return $return;

	$dts = $context['timezones']['DTS'][$context['timezones']['TZ'][$zone]['rule']];
	$utcSeconds = $context['timezones']['TZ'][$zone]['seconds'];

	//Let's detect which is the current time :)
	$utcTimestamp = (int) $utcTimestamp;
	$utcTimestamp = $utcTimestamp >= 0 ? $utcTimestamp : 0;
	$currentTime = !empty($utcTimestamp) ? $utcTimestamp : utcTimestamp();
	$year = date("Y", $currentTime);

	//This Year
	$rule_0 = dtsTimestamp($dts['0_in'], $dts['0_on'], $dts['0_at_hour'] ,$dts['0_at_min'], $dts['0_at_type'], $dts['1_seconds'], $utcSeconds, $year);
	$rule_1 = dtsTimestamp($dts['1_in'], $dts['1_on'], $dts['1_at_hour'] ,$dts['1_at_min'], $dts['1_at_type'], $dts['0_seconds'], $utcSeconds, $year);

	if($rule_0 === false || $rule_1 === false)
	{
		trigger_error('calculateZoneDtsData(), ('.$dts['rule'].') DTS error, mistake in database!');
		return $return;
	}

	//Now this is difficult to decripe, i need to detect the correct order...
	$rule_order = $rule_0 < $rule_1;
	$rules = array(
		'0' => $rule_order ? $rule_0 : $rule_1,
		'1' => $rule_order ? $rule_1 : $rule_0,
	);

	if($rules[0] > $currentTime)
		$select = 0;
	elseif($rules[1] > $currentTime)
		$select = 1;
	//Oh look for the next year?
	else
	{
		//Next Year
		$year++;
		$rule_2 = dtsTimestamp($dts['0_in'], $dts['0_on'], $dts['0_at_hour'] ,$dts['0_at_min'], $dts['0_at_type'], $dts['1_seconds'], $utcSeconds, $year);
		$rule_3 = dtsTimestamp($dts['1_in'], $dts['1_on'], $dts['1_at_hour'] ,$dts['1_at_min'], $dts['1_at_type'], $dts['0_seconds'], $utcSeconds, $year);

		//Now this is difficult to decripe, i need to detect the correct order...
		$rule_order = $rule_2 < $rule_3;
		$rules = array(
			'0' => $rule_order ? $rule_2 : $rule_3,
			'1' => $rule_order ? $rule_3 : $rule_2,
		);
		if($rules[0] > $currentTime)
			$select = 0;
		elseif($rules[1] > $currentTime)
			$select = 1;
		//This could normal not happen, but if than i can not callucalate a timestamp oO
		else
			return $return;
	}

	//Okay take the correct
	$return['nextUpdate'] = $rules[$select];
	$var = ($select == 1 ? 0 : 1).'_seconds';
	$return['saveSeconds'] = $dts[$var];
	$return['saveHour'] = $dts[$var] / 3600;
	$return['offsetSeconds'] = $context['timezones']['TZ'][$zone]['seconds'] + $dts[$var];
	$return['offsetHours'] = $context['timezones']['TZ'][$zone]['hours'] + ($dts[$var] / 3600);

	return $return;
}

function dtsTimestamp($in, $on, $hour, $min, $type, $last_seconds, $utcSeconds, $year = false)
{
	static $month2n, $day2n;

	if(empty($on))
		return false;

	//A Little bit prepareing :X
	if(!isset($month2n))
	{
		$month2n = array(
			'Jan' => 1,
			'Feb' => 2,
			'Mar' => 3,
			'Apr' => 4,
			'May' => 5,
			'Jun' => 6,
			'Jul' => 7,
			'Aug' => 8,
			'Sep' => 9,
			'Oct' => 10,
			'Nov' => 11,
			'Dec' => 12,
		);
		$day2n = array(
			'Sun' => 0,
			'Mon' => 1,
			'Tue' => 2,
			'Wed' => 3,
			'Thu' => 4,
			'Fri' => 5,
			'Sat' => 6,
		);
	}

	//Use the correct Year for this
	if($year == 'next' || $year == '+')
	{
		$year = date('Y', utcTimestamp());
		$year++;
	}
	elseif($year == 'prev' || $year == '-')
	{
		$year = date('Y', utcTimestamp());
		$year--;
	}
	elseif(empty($year) || $year == false)
	{
		$year = date('Y', utcTimestamp());
	}
	else {
		$year = (int) $year;
		if(empty($year))
			$year = date('Y', utcTimestamp());
	}

	//Callculate the this timestamp, it's based on the utcTimestamp!
	//w = watchtime, s = standardtime (without DTS), u = utc time based
	$in = $month2n[$in];

	if(substr($on, 0, 4) == 'last')
	{
		$day = $day2n[substr($on, 4, 3)];
		//Need to be an dts untouched time... php bug fix on server with dts!
		$lastDay = mktime(11,11,11,($in+1), 0, $year);
		for($x=0; $x<=6; $x++)
		{
			$current = $lastDay-($x*3600*24);
			if(date("w", $current) == $day)
			{
				$on = date("j", $current);
				break;
			}
		}
	}
	elseif(!is_numeric($on))
	{
		$data = strpos($on, '=>') !== false ? explode('=>', $on) : explode('>=', $on);
		$day = $day2n[$data[0]];
		while(true)
		{
			if(!isset($data[1]))
			{
				trigger_error('dtsTimestamp(), ('.implode('', $data).') split error, mistake in database!');
				return false;
			}
			$ct = mktime(0,0,0,$in,$data[1],$year);
			if(date("w", $ct) == $day)
			{
				$on = date("j", $ct);
				break;
			}
			else
				$data[1]++;
		}
	}

	//utc midnight :D
	$utc = mktime(0,0,0,$in,$on,$year);

	//Time Callculation 8)
	$sec = $hour * 3600 + $min * 60;
	//$secs are callculated to the $utc ;)
	//u = Utc based, s = standard time (without daylightsaving), anyting else = current watchtime :)
	if($type == 'u')
		$utc += $sec;
	elseif($type == 's')
		$utc = $utc + $sec + $utcSeconds;
	else
		$utc = $utc + $sec + $utcSeconds + $last_seconds;

	return $utc;
}

//This Function return a userbased Timestamp or a forum based timestamp *g*
function timezone_timestamp($use_user_offset = true, $timestamp = null)
{
	global $user_info, $modSettings;

	if ($timestamp === null)
		$timestamp = time();
	elseif ($timestamp <= 0)
		$timestamp = 0;

	$utcTimestamp = utcTimestamp($timestamp);

	//Return the Userbased Time Offset?
	if($use_user_offset && !empty($user_info['timezone']))
		return $utcTimestamp + $user_info['timezone_offset'];
	//Return the Standard Time Offset?
	elseif(!empty($modSettings['timezones_useStandard']) && !empty($modSettings['timezones_standard']))
	{
		//Update Needed?
		if(!empty($modSettings['timezones_standard_update']) && $modSettings['timezones_standard_update'] < $timestamp)
			update_standardTimeZone();

		if(empty($modSettings['timezones_standard_noFix'])) //Server and Standard are the same :)
			return $timestamp;

		//Okay a little bit faster and sometimes 0 is not set ;D
		if(empty($modSettings['timezones_standard_offset']))
			return $utcTimestamp;
		else
			return $utcTimestamp + $modSettings['timezones_standard_offset'];
	}
	//Okay only the Standardoffset :X (Hope you know what you do xD)
	elseif(!empty($modSettings['timezones_enable']))
		return $timestamp + ($modSettings['time_offset'] * 3600);
	//Okay the SMF Way ;)
	else
		return $timestamp + ($modSettings['time_offset'] + ($use_user_offset ? $user_info['time_offset'] : 0)) * 3600;
}

function update_standardTimeZone()
{
		global $modSettings;

		if(empty($modSettings['timezones_standard']))
			return;

		$zoneCalc = calculateZoneDtsData($modSettings['timezones_standard']);
		//Let's find out if the Standard i the same as the Standardtime, it's for speed reasons ;)
		$check = time();
		$check2 = zoneTimestamp($check, $modSettings['timezones_standard']);
		$settingsUpdate = array(
			'timezones_standard_update' => $zoneCalc['nextUpdate'],
			'timezones_standard_offset' => $zoneCalc['offsetSeconds'],
			'timezones_standard_noFix' => $check == $check2 ? '0' : '1',
		);
		updateSettings($settingsUpdate);
}

function update_memberTimeZone($memID, $timezone)
{
		global $modSettings, $user_info, $user_profile, $ID_MEMBER;

		$zoneCalc = calculateZoneDtsData($timezone);
		//Let's find out if the Standard i the same as the Standardtime, it's for speed reasons ;)
		$membersUpdate = array(
			'timezone' => "'$timezone'",
			'timezone_update' => $zoneCalc['nextUpdate'],
			'timezone_offset' => $zoneCalc['offsetSeconds'],
		);
		updateMemberData(array($memID), $membersUpdate);

		//Fix some loaded things 8)
		if(isset($user_profile[$memID]))
		{
			$user_profile[$memID]['timezone'] = $timezone;
			$user_profile[$memID]['timezone_update'] = $zoneCalc['nextUpdate'];
			$user_profile[$memID]['timezone_offset'] = $zoneCalc['offsetSeconds'];
			global $memberContext;
			if(isset($memberContext[$memID]))
			{
				$memberContext[$memID]['timezone'] = $timezone;
				$memberContext[$memID]['timezone_update'] = $zoneCalc['nextUpdate'];
				$memberContext[$memID]['timezone_offset'] = $zoneCalc['offsetSeconds'];
			}
		}
		if($ID_MEMBER == $memID)
		{
			$user_info['timezone'] = $timezone;
			$user_info['timezone_update'] = $zoneCalc['nextUpdate'];
			$user_info['timezone_offset'] = $zoneCalc['offsetSeconds'];
		}
}

//This load the Local Time off this user, it also make a checkup if the offsets are correct and fix them.
function local_userTime($memID)
{
		global $modSettings, $user_info, $user_profile, $ID_MEMBER;

		if(!isset($user_profile[$memID]) && !empty($memID))
			loadMemberData($memID);

		//Current local time calculation 8)
		$time = time();
		
		// Prepare the timezone
		$timezone = '%Z';
		if(strpos($user_info['time_format'], '%Z') !== false)
		{
			$zone = (!empty($user_profile[$memID]['timezone']) ? $user_profile[$memID]['timezone'] : (!empty($modSettings['timezones_useStandard']) ? $modSettings['timezones_standard'] : ''));
			$offset = (!empty($user_profile[$memID]['timezone']) ? $user_profile[$memID]['timezone_offset'] : (!empty($modSettings['timezones_useStandard']) ? $modSettings['timezones_standard_offset'] : 0));
			$timezone = return_shortTimezone($zone, $offset);
		}

		// Okay if not timezone_offset is set take standard :P
		$offset = (!empty($user_info['timezone']) ? $user_info['timezone_offset'] : (!empty($modSettings['timezones_useStandard']) ? $modSettings['timezones_standard_offset'] : 0));		

		//The user don't use any timezone, so it's the forumstandard 8)
		if(empty($memID) || empty($user_profile[$memID]['timezone']))
		{
			$standardOffset = !empty($modSettings['timezones_useStandard']) ? $modSettings['timezones_standard_offset'] : 0;
			$calculated = empty($modSettings['timezones_useStandard']) ? utcTimestamp() : time() + ($modSettings['timezones_standard_offset'] - $offset);
			return timeformat($calculated, false, $timezone);
		}

		if(!empty($user_profile[$memID]['timezone_update']) && $user_profile[$memID]['timezone_update'] < utcTimestamp())
			update_memberTimeZone($memID, $user_profile[$memID]['timezone']);
		
		return timeformat($time + ($user_profile[$memID]['timezone_offset'] - $offset), false, $timezone);
}

function return_shortTimezone($zone = '', $currentOffset = 0)
{
	static $shortzone = array();

	// Return standard on nothing :P
	if (empty($zone))
		return '%Z';
	elseif (!empty($shortzone[$zone]))
		return $shortzone[$zone];
	
	// Okay do some work :)
	global $context, $db_prefix;

	$szone = array(
		'short_zone' => '%Z',
		'dts' => 0,
		'seconds' => 0,
	);
	if (!empty($context['timezones']['TZ'][$zone]))
		$szone = array(
			'short_zone' => $context['timezones']['TZ'][$zone]['short_zone'],
			'dts' => $context['timezones']['TZ'][$zone]['dts'],
			'seconds' => $context['timezones']['TZ'][$zone]['seconds'],
		);
	else
	{
		// Okay take the short zone and the time :D
		$request = db_query("
			SELECT short_zone, dts, seconds
			FROM {$db_prefix}timezone
			WHERE zone = '$zone'
			LIMIT 1", __FILE__, __LINE__);
		$data = mysql_fetch_assoc($request);
		mysql_free_result($request);
		if (!empty($data))
			$szone = $data;
	}
	
	// Let's see if there is a dts zone this is a special case :D
	if (empty($szone['dts']))
		$shortzone[$zone] = $szone['short_zone'];
	else
	{
		$szone['short_zone'] = explode('/', $szone['short_zone']);
		// Okay sorry i can't detect it...
		if (count($szone['short_zone']) > 2)
			$shortzone[$zone] = implode('/', $szone['short_zone']);
		// Okay no diffrence?
		elseif (count($szone['short_zone']) == 1 || $szone['short_zone'][0] == $szone['short_zone'][1])
			$shortzone[$zone] = current($szone['short_zone']);
		// The current Timezone of the user can help me :D
		elseif (!empty($currentOffset))
		{
			if ($currentOffset == $szone['seconds'])
				$shortzone[$zone] = $szone['short_zone'][1];
			else
				$shortzone[$zone] = $szone['short_zone'][0];
		}
		// The hard way... load the complete timezone data... with this i can exact say
		else
		{
			$dts = calculateZoneDtsData($zone);
			if ($dts['offsetSeconds'] == $szone['seconds'])
				$shortzone[$zone] = $szone['short_zone'][1];
			else
				$shortzone[$zone] = $szone['short_zone'][0];
		}
	}
	return $shortzone[$zone];
}

function return_selectField($tz_data)
{
	global $modSettings;
	if(empty($modSettings['timezones_selectField']))
		updateSettings(array('timezones_selectField' => '%country (%city_land, %designation GMT%offset)'));

	$replace = array(
		'%name' => $tz_data['replaceName'],
		'%zone' => $tz_data['zone'],
		'%country' => $tz_data['country_name'],
		'%designation' => $tz_data['short_zone'],
		'%offset' => $tz_data['utc'],
		'%timezone' => $tz_data['zone'],
		'%worldpart' => $tz_data['master_zone'],
		'%ccode' => $tz_data['iso_code'],
		'%dst' => !empty($tz_data['dts']) ? 'DST' : '',
		'%city_land' => $tz_data['sub_zone'],
		'%comment' => $tz_data['comment'],
	);

	return str_replace(array_keys($replace), array_values($replace), $modSettings['timezones_selectField']);
}

?>